/* Routines to maintain a list of online servers.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include "services.h"

Server *server_list;

/*************************************************************************/
/*************************************************************************/

static Server *server_add(char const *name, Server *hub)
{
	Server *serv;
	
	serv = scalloc(1, sizeof(Server));
	strscpy(serv->name, name, SERVERMAX);
	
	if (hub) {
		serv->hub = hub;
		serv->hub->leafcount++;
	}
	
	/* WARNING!
	 * don't change this code because server_delete relies
	 * on it: a leaf must be PRIOR its hub in the list! */
	serv->next = server_list;
	if (server_list)
		server_list->prev = serv;
	server_list = serv;
	/* END OF WARNING */
	
	if (debug)
		slog("debug: new server: %s (hub = %s)", serv->name, 
				(serv->hub ? serv->hub->name : "none"));
	
	return serv;
}

/*************************************************************************/

static void server_delete(Server *serv)
{
#ifdef HAS_NOQUIT
	int i;
	char *av[1] = { "Netsplit." };
	Server *serv2, *s_next;
	User *user, *u_next;
#endif

	if (debug)
		slog("debug: deleting server: %s (leafcount = %d)", 
				serv->name, serv->leafcount);

#ifdef HAS_NOQUIT

	/* This loop assumes that servers linked to serv2
	 * are always BEFORE serv2 in the list. This is possible
	 * because of the order of the SERVER messages. */
	for (serv2 = server_list; serv2; serv2 = s_next) {
		s_next = serv2->next;
		
		if (serv2->hub == serv)
			server_delete(serv2);
	}
	
	for (i = 0; i < 1024; i++) {
		for (user = userlist[i]; user; user = u_next) {
			u_next = user->next;
			
			if (user->server == serv)
				do_quit(user->nick, 1, av);
		}
	}
#endif

	if (serv->hub)
		serv->hub->leafcount--;

	if (serv->next)
		serv->next->prev = serv->prev;
	if (serv->prev)
		serv->prev->next = serv->next;
	else
		server_list = serv->next;
	
	free(serv);
}

/*************************************************************************/

Server *server_find(char const *name)
{
	Server *serv;
	
	for (serv = server_list; serv; serv = serv->next)
		if (!stricmp(serv->name, name))
			return serv;
	
	return NULL;
}

/*************************************************************************/
/*************************************************************************/

void do_server(char const *source, int ac, char **av)
{
	Server *serv;
	
	serv = server_add(av[0], (source ? server_find(source) : NULL));
	
	serv->hopcount = atoi(av[1]);
	strscpy(serv->desc, av[2], SDESCMAX);
}

/*************************************************************************/

void do_squit(char const *source, int ac, char **av)
{
	Server *serv = server_find(av[0]);
	
	if (serv)
		server_delete(serv);
#ifdef IRC_BAHAMUT
	else
		/* Probably a juped server, needs to send an SQUIT
		 * message back to comply with the UNCONNECT feature. */
		send_cmd(ServerName, "SQUIT %s :%s", av[0], source);
#endif
}

/*************************************************************************/
